<?php

//  po-view.php - a web viewer for PO files
//  Copyright (C) 2004,2005 Sergey Poznyakoff
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2, or (at your option)
//  any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA

 $po_dir="/home/gray/public_html/test/po/";
 $base_url="/test/po/";

 $text = array();
 $line_count = 0;
 $fuzzy_count = 0;
 $entry_count = 0;
 $xlate_count = 0;

 function first_entry_msgstr ($line, &$input_charset)
 {
   global $project_id, $language;

   if (preg_match ('/Content-Type: .*; charset=([a-zA-Z_\-0-9\.]*)/i',
                   $line, $matches))
     $input_charset = $matches[1];
   else if (preg_match ('/Project-Id-Version: (.*)\\\/i', $line, $matches)) 
     $project_id = $matches[1];
   else if (preg_match ('/Language-Team: ([^ ]+)/i', $line, $matches))
     $language = $matches[1];
 }

 # the three states:
 #  0 - initial
 #  1 - msgid found. Expected msgstr or ""
 #  2 - msgstr found, entry is resolved. Expected is an empty line to switch
 #      to 0
 #  3 - empty msgstr found. Expected is another msgstr or empty line
 #

 function scan ($filename)
 {
   global $text, $line_count, $fuzzy_count, $xlate_count, $entry_count;

   $state = 0;

   $input_charset = "UTF-8";
   foreach (file ("$filename") as $line)
     {
       $text[$line_count++] = iconv ($input_charset, "UTF-8", $line);

       switch ($state)
         {
           case 0: // Initial state
             if (preg_match ('/^msgid/', $line))
               {
                 $state = 1;
                 $entry_count++;
               }
             else if (preg_match ('/^#.*, fuzzy/', $line))
               $fuzzy_count++;
             break;

           case 1:
             if (preg_match ('/^msgstr/', $line))
               {
	         $s = preg_replace ('/\\\"/', '{QUOTE}', $line);

                 if (preg_match ('/^msgstr  *"([^"]+)"/', $s)
                     || preg_match ('/^msgstr\[0\]  *"([^"]+)"/', $s))
                   {
                     $xlate_count++;
                     $state = 2;
                   }
                 else 
                   $state = 3;

                 if ($entry_count == 1)
	           first_entry_msgstr (preg_replace ('/msgstr  *"(.*)"/', '$1',
                                                     $line), $input_charset);
               }
             break;

           case 2:
             if (preg_match ('/^ *$/', $line)) 
               $state = 0;
             else if ($entry_count == 1) 
               first_entry_msgstr ($line, $input_charset);
             break;

           case 3:
             if (preg_match ('/^"([^"]+)*"/', $line))
               {
                 if ($entry_count == 1) 
                   first_entry_msgstr ($line, $input_charset);
                 $xlate_count++;
                 $state = 2;
               }
             else if (preg_match ('/^ *$/', $line)) 
               $state = 0;
         }    
//       echo "LINE $line COUNT " . $entry_count . "\n";

    }
 }


 $html_pattern = array (
  '/<([^@]*)>/',
  '/<([^>]*)@(.*)>/',
  '//');

 $html_replacement = array (
  '&lt;$1&gt;',
  '&lt;<a href="mailto:$1@$2">$1 at foobar dot com</a>&gt;',
  '<span class="spec">^K</span>');

 function fixup ($line)
 {
   global $html_pattern, $html_replacement;
   return preg_replace ($html_pattern, $html_replacement, $line);
 }

 function print_stats ()
 {
   global $line_count, $fuzzy_count, $xlate_count, $entry_count;

   print "<h2>Statistics</h1>";
   print "<table>";
   print "<tr><td>Translated:</td><td align=\"right\">" . $xlate_count . "</td></tr>";
   print "<tr><td>Fuzzy:</td><td align=\"right\">" . $fuzzy_count . "</td></tr>";
   print "<tr><td>Untranslated:</td><td align=\"right\">" . ($entry_count-$xlate_count) . "</td></tr>";
   print "<tr><td colspan=\"2\"><hr></td></tr>";
   print "<tr><td><b>Total</b></td><td align=\"right\">" . $entry_count . "</td></tr>";
   print "</table>";
 }
?>
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN">
<html>
<head>
<title><?php
  $project_id = "Unknown Project";
  $language = "Unknown language";
  if (isset ($_REQUEST['lang']))
    {
      if (isset ($_REQUEST['proj']))
        $fn = $_REQUEST['proj'] . "/" . $_REQUEST['lang'] . ".po";
      else
        $fn = $_REQUEST['lang'] . ".po";

      $download_url = $base_url . $fn;
      $fn = $po_dir . $fn;
      if (file_exists ($fn))
        {
          scan ($fn);
          $title = $language . " PO file for $project_id: " . $_REQUEST['lang'];
        }
    }
  else
    $title = "Untitled";

  print ($title);
?></title>
 <link rev="made" href="mailto:gray@mirddin.farlep.net" />
 <meta http-equiv="content-type" content="text/html; charset=utf-8" />
 <meta name="robots" content="index,nofollow" />
 <meta name="generator" content="po-view.php" />
 <link rel="stylesheet" type="text/css" href="/test/po.css" />
</head>
<body id="top">
<?
  $printf_regex='/(%([#0\- +I][1-9]*(\.[0-9]*)?([hlLqjzt][hl]?)?)?[diouxXeEfFgGaAcsCSpn%])/';
  $printf_replace='<span class="printf">$1</span>';

  define("STATE_INITIAL",0);
  define("STATE_MSGID",1);
  define("STATE_MSGSTR",2);
  define("STATE_COMMENT",3);

  $state=STATE_INITIAL;

  function set_state($new_state)
  {
    global $state;

    if ($state != $new_state)
      {
        if ($new_state == STATE_INITIAL)
          print '<br />';
        $state = $new_state;
      }
  }

  function print_line($line)
  {
    global $state, $printf_regex, $printf_replace;

    if (preg_match ('/^#/', $line))
      {
         set_state (STATE_COMMENT);
         print "<div class=\"comment\">$line</div>";
      }
    else if (preg_match ('/^msgid/', $line))
      {
         set_state (STATE_MSGID);
         print preg_replace($printf_regex,
                            $printf_replace,
                            preg_replace('/(.*)(msgid(_plural)?) *"(.*)"$/',
           '$1<div class="msgid">$2 "<span class="orig">$4</span>"</div>',
                                         $line));
      }
    else if (preg_match ('/^msgstr/', $line))
      {
         set_state (STATE_MSGSTR);
         print preg_replace($printf_regex,
                            $printf_replace,
                            preg_replace('/(.*)(msgstr(\[[0-9]\])?) *"(.*)"$/',
        '$1<div class="msgstr">$2 "<span class="trans">$4</span>"</div>',
                                         $line));
      }
    else if (preg_match ('/ *".*"$/', $line))
      {
         print preg_replace($printf_regex,
                         $printf_replace,
                         preg_replace('/( *)"(.*)"$/',
                          '$1<div class="'. ($state == STATE_MSGSTR ? 'msgstr' : 'msgid') .
                          '">"<span class="'. ($state == STATE_MSGSTR ? 'trans' : 'orig') .
                          '">$2</span>"</div>',
                          $line));
      }
    else
      {
        set_state (STATE_INITIAL);
        print $line;
      } 
  }

  function print_download()
  {
    global $download_url;
    print '<a href="' . $download_url . '">download</a>';
  }

  if ($line_count > 0)
    {
      print_stats ();
      print_download ();
      print "<hr>";
      print "<h2>Text</h2>";

      foreach ($text as $line)
        print_line (fixup ($line));
    }
?>
<hr />
<div class="footer" align="center">
[ <a href="#top">Return to the beginning</a> | <?php print_download() ?> ]
</div>
</body>
</html>
